from syned.beamline.shape import SurfaceShape, BoundaryShape
from syned.beamline.optical_element_with_surface_shape import OpticalElementsWithSurfaceShape

class Grating(OpticalElementsWithSurfaceShape):
    def __init__(self,
                 name="Undefined",
                 surface_shape=SurfaceShape(),
                 boundary_shape=BoundaryShape(),
                 ruling = 800e3
                 ):
        super().__init__(name, surface_shape, boundary_shape)
        self._ruling = ruling

        # support text containg name of variable, help text and unit. Will be stored in self._support_dictionary
        self._set_support_text([
                    ("name",                "Name" ,                  "" ),
                    ("surface_shape",       "Surface Shape" ,         "" ),
                    ("boundary_shape",      "Boundary Shape" ,        "" ),
                    ("ruling",              "Ruling at center" ,      "lines/m^3" ),
            ] )




"""
Base class for LighSource, which contains:
    - a name
    - an electron beam
    - a magnetic structure

"""
from syned.syned_object import SynedObject
from syned.storage_ring.magnetic_structure import MagneticStructure
from syned.storage_ring.electron_beam import ElectronBeam

class LightSource(SynedObject):
    def __init__(self, name="Undefined", electron_beam=ElectronBeam(), magnetic_structure=MagneticStructure()):
        self._name               = name
        self._electron_beam      = electron_beam
        self._magnetic_structure = magnetic_structure
        # support text containg name of variable, help text and unit. Will be stored in self._support_dictionary
        self._set_support_text([
                    ("name",              "Name",""),
                    ("electron_beam",     "Electron Beam",""),
                    ("magnetic_structure","Magnetic Strtructure",""),
            ] )


    def get_name(self):
        return self._name

    def get_electron_beam(self):
        return self._electron_beam

    def get_magnetic_structure(self):
        return self._magnetic_structure



"""
Implement an undulator with vertical and horizontal magnetic fields.
"""
import numpy as np
import scipy.constants as codata

cte = codata.e/(2*np.pi*codata.electron_mass*codata.c)

from syned.storage_ring.magnetic_structures.insertion_device import InsertionDevice

class Undulator(InsertionDevice):

    def __init__(self,
                 K_vertical = 0.0,
                 K_horizontal = 0.0,
                 period_length = 0.0,
                 number_of_periods = 1):
        InsertionDevice.__init__(self, K_vertical, K_horizontal, period_length, number_of_periods)

    def resonance_wavelength(self, gamma, theta_x=0.0, theta_z=0.0, harmonic=1):
        wavelength = (self.period_length() / (2.0*gamma **2)) * \
                     (1 + self.K_vertical()**2 / 2.0 + self.K_horizontal()**2 / 2.0 + \
                      gamma**2 * (theta_x**2 + theta_z ** 2))

        return wavelength/harmonic

    def resonance_frequency(self, gamma, theta_x=0.0, theta_z=0.0, harmonic=1):
        frequency = codata.c / self.resonance_wavelength(gamma, theta_x, theta_z)

        return frequency*harmonic

    def resonance_energy(self, gamma, theta_x=0.0, theta_z=0.0, harmonic=1):
        energy_in_ev = codata.h * self.resonance_frequency(gamma, theta_x, theta_z) / codata.e

        return energy_in_ev*harmonic

    def gaussian_central_cone_aperture(self, gamma, n=1):
        return (1/gamma)*np.sqrt((1.0/(2.0*n*self.number_of_periods())) * (1.0 + self.K_horizontal()**2/2.0 + self.K_vertical()**2/2.0))

    @classmethod
    def initialize_as_vertical_undulator(cls, K = 0.0, period_length = 0.0, periods_number = 1):
        return Undulator(K_vertical=K,
                         K_horizontal=0.0,
                         period_length=period_length,
                         number_of_periods=periods_number)

if __name__ == "__main__":

    a = Undulator(number_of_periods=61.5, period_length=0.057)
    a.set_K_vertical_from_magnetic_field(0.187782)

    print(a._K_vertical)

    print (a.resonance_energy(gamma=5870.8540997356595))

    fd = a.to_full_dictionary()
    dict = a.to_dictionary()

    print(dict)

    for key in fd:
        print(key,fd[key][0])

    for key in fd:
        print(key,dict[key])

    print(a.keys())
    print(a.info())




# THIS FILE IS GENERATED FROM syned SETUP.PY
short_version = '1.0.13'
version = '1.0.13'
full_version = '1.0.13.dev0+f843317'
git_revision = 'f84331781f6e66d58681b3248a86a2a79b5bd384'
release = False

if not release:
    version = full_version
    short_version += ".dev"



"""
Base class for a Bending Magnet


"""
from syned.storage_ring.magnetic_structure import MagneticStructure
import scipy.constants as codata

class BendingMagnet(MagneticStructure):
    def __init__(self, radius, magnetic_field, length):
        """
        Constructor.
        :param radius: Physical Radius/curvature of the magnet in m
        :param magnetic_field: Magnetic field strength in T
        :param length: physical length of the bending magnet (along the arc) in m.
        """
        MagneticStructure.__init__(self)
        self._radius         = radius
        self._magnetic_field = magnetic_field
        self._length         = length

        # support text containg name of variable, help text and unit. Will be stored in self._support_dictionary
        self._set_support_text([
                    ("radius"          , "Radius of bending magnet" , "m"    ),
                    ("magnetic_field"  , "Magnetic field",            "T"    ),
                    ("length"          , "Bending magnet length",     "m"   ),
            ] )

    #
    #methods for practical calculations
    #

    def length(self):
        return self._length

    def magnetic_field(self):
        return self._magnetic_field

    def radius(self):
        return self._radius

    def horizontal_divergence(self):
        return self.length()/self.radius()

    def get_magnetic_field(self, electron_energy_in_GeV):
        return BendingMagnet.calculate_magnetic_field(self._radius, electron_energy_in_GeV)

    def get_magnetic_radius(self, electron_energy_in_GeV):
        return BendingMagnet.calculate_magnetic_radius(self._magnetic_field, electron_energy_in_GeV)

    def get_critical_energy(self, electron_energy_in_GeV):
        return BendingMagnet.calculate_critical_energy(self._radius, electron_energy_in_GeV)

    # for equations, see for example https://people.eecs.berkeley.edu/~attwood/srms/2007/Lec09.pdf
    @classmethod
    def calculate_magnetic_field(cls, magnetic_radius, electron_energy_in_GeV):
        # return 3.334728*electron_energy_in_GeV/magnetic_radius
        gamma = 1e9 * electron_energy_in_GeV / (codata.m_e *  codata.c**2 / codata.e)
        # B = gamma m c / e / R ; note that c approximated the electron velocity!
        return gamma * codata.m_e * codata.c / codata.e / magnetic_radius

    @classmethod
    def calculate_magnetic_radius(cls, magnetic_field, electron_energy_in_GeV):
        # return 3.334728*electron_energy_in_GeV/magnetic_field
        gamma = 1e9 * electron_energy_in_GeV / (codata.m_e *  codata.c**2 / codata.e)
        # R = gamma m c / e / B ; note that c approximated the electron velocity!
        return gamma * codata.m_e * codata.c / codata.e / magnetic_field

    @classmethod
    def calculate_critical_energy(cls, magnetic_radius, electron_energy_in_GeV):
        # omega = 3 g3 c / (2r)
        gamma = 1e9 * electron_energy_in_GeV / (codata.m_e *  codata.c**2 / codata.e)
        critical_energy_J = 3 * codata.c * codata.hbar * gamma**3 / (2 * magnetic_radius)
        critical_energy_eV = critical_energy_J / codata.e
        return critical_energy_eV

if __name__ == "__main__":
    print("input for ESRF: ")
    print(">> B = ",BendingMagnet.calculate_magnetic_field(25.0,6.04))
    print(">> R = ",BendingMagnet.calculate_magnetic_radius(0.85,6.04))
    print(">> Ec = ",BendingMagnet.calculate_critical_energy(23.7,6.04))
